/*
 * Since this version of xv6 uses paging, we've adopted the convention
 * of linking the kernel in the upper gigabyte of virtual memory. The
 * kernel is loaded, however, down at 0x100000: the second megabyte of
 * physical memory. Before we can start executing kernel code, then,
 * we need to fix up the segment registers so that linear addresses
 * translate down into low memory. So:
 */

#include <asm.h>
#include <param.h>
#include <page.h>

/* RELOC(x) subtracts the kernel offset from X to the (current) linear address*/
#define RELOC(x) ((x) - KERNEL_OFFSET)

.set CODE_SEL,0x8
.set DATA_SEL,0x10

.text

.globl _start
_start:

    # the new GDT (see below) has the appropriate segment descriptors
    # to accomplish the linear address mapping we need pre-paging
    lgdt    RELOC(prepage_gdtdesc)

    # reload segment registers
    movl    $DATA_SEL, %eax
    movw    %ax,%ds
    movw    %ax,%es
    movw    %ax,%ss
    ljmp    $CODE_SEL,$dummy
dummy:
    # Going to C soon, where we'll have frame pointers. Clearing
    # ebp here will allow stack traces to terminate correctly. 
    movl    $0x0,%ebp

    # set up stack for transfer to C
    movl    $bootstacktop,%esp

    call    main

spin:
    jmp     spin

/*
 * A page for the kernel page directory
 */
.section ".bss.page_aligned","w"
    .p2align    PAGE_SHIFT
    .globl kernel_pgd
kernel_pgd:
    .fill 1024,4,0

.data

/* The boot stack */
    .p2align    PAGE_SHIFT
    .globl bootstack 
bootstack:
    .space KSTACKSIZE
    .globl bootstacktop
bootstacktop:

/* pre-paging GDT */
    .p2align    2
prepage_gdt:
    SEG_NULLASM
    SEG_ASM(STA_X|STA_R, -KERNEL_OFFSET, 0xffffffff)   # code
    SEG_ASM(STA_W, -KERNEL_OFFSET, 0xffffffff)   # data
prepage_gdtdesc:
    .word   0x17
    .long   RELOC(prepage_gdt)


    
