#ifndef _LIST_H_
#define _LIST_H_

#include "poison.h"     // bad pointer defs

/* 
 * This is a doubly linked list definition derived from the Linux
 * kernel source code. These lists are a common OS idiom for enabling
 * various data structures to be efficiently added and removed from
 * lists using a generic interface.
*/

#ifndef __ASSEMBLER__

struct list_head {
    struct list_head *next, *prev;
};

/* Initialize a list_head with itself as next, prev */
#define LIST_HEAD_INIT(l) { &(l), &(l) }
#define LIST_HEAD(l) \
    struct list_head l = LIST_HEAD_INIT(l)

/* Same for resetting an existing list_head */
static inline void INIT_LIST_HEAD(struct list_head *l)
{
    l->next = l;
    l->prev = l;
}

static inline void __list_add(struct list_head *new, 
    struct list_head *prev,
    struct list_head *next)
{
    next->prev = new;
    new->next = next;
    new->prev = prev;
    prev->next = new;
}

/* Insert after given head (front of list) */
static inline void list_add(struct list_head *new, struct list_head *head)
{
    __list_add(new,head,head->next);
}

/* Insert at tail of list */
static inline void list_add_tail(struct list_head *new, struct list_head *head)
{
    __list_add(new, head->prev, head);
}

/* removal */
static inline void __list_del(struct list_head *prev, struct list_head *next)
{
    prev->next = next;
    next->prev = prev;
}

static inline void list_del(struct list_head *entry)
{
    __list_del(entry->prev,entry->next);
    entry->next = BAD_POINTER;      // ease debugging
    entry->prev = BAD_POINTER;
}

/* removal and re-initialization */
static inline void list_del_init(struct list_head *entry)
{
    __list_del(entry->prev, entry->next);
    INIT_LIST_HEAD(entry);
}

static inline int list_empty(const struct list_head *head)
{
    return head->next == head;
}

/* 
 * Returns the structure containing this list entry (ptr)
 * type: structure type
 * member: name of list member of structure
 */
#define offsetof(TYPE, MEMBER) ((unsigned int) &((TYPE *)0)->MEMBER)
#define list_entry(ptr, type, member) ({          \
        const typeof( ((type *)0)->member ) *__mptr = (ptr);    \
        (type *)( (char *)__mptr - offsetof(type,member) );})


#endif
#endif
